/* example.cpp
 * Exemple d'utilisation du code de base permettant de crer
 * une fentre OpenGL sous Windows avec les routines WGL.
 * Touches:
 *	F1 - Mode plein cran ou fentr
 *	ESC - Quitter
 *
 * Auteur: Vouriot Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 11 septembre 2002
 */

#include "initWGL.h"

#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens

#define TITLE "Example BaseCodeGL (Vouriot Thierry)"	// titre du programme

GL_Window*	g_window;	// fentre OpenGL
Keys*		g_keys;		// touches clavier

/* Initialisation de l'utilisateur et d'OpenGL */
bool Initialize(GL_Window* window, Keys* keys)
{
	g_window = window;
	g_keys = keys;
	SetWindowText(g_window->hWnd, TITLE);

	// OpenGL initialisation
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glShadeModel(GL_SMOOTH);
	
	return true;												// Return TRUE (Initialization Successful)
}

/* Appele juste avant la destruction de la fentre,
 * Libration mmoire, ...etc... 
 */
void Deinitialize(void)
{
	// rien  faire
}

/* Appele avant chaque affichage d'image,
 * Teste des touches clavier,
 * Calcule du frame rate  (milliseconds est le temps entre 2 affichages d'images),
 * Maj des diffrentes variables, ...etc...
 */
void Update(DWORD milliseconds)
{
	// teste des touches du clavier
	if (g_keys->keyDown[VK_ESCAPE] == true)	// echap on quitte l'application
	{
		TerminateApplication(g_window);
	}
	if (g_keys->keyDown[VK_F1] == true)	// F1 basculement fullscreen/windowed
	{
		g_keys->keyDown[VK_F1] = false;
		ToggleFullscreen(g_window);
	}
}

/* Affichage de la scne */
void DisplayScene(void) 
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	// 	  Position      View	   Up Vector
	gluLookAt(0, 0, 5,     0, 0, 0,     0, 1, 0);
	
	// on dessine un simple triangle
	// avec 3 sommets de couleurs diffrents
	glBegin(GL_TRIANGLES);
		glColor3f(1.0, 0.0, 0.0);	// rouge
		glVertex3f(-1.5, -1.0, 0.0);
		glColor3f(0.0, 1.0, 0.0);	// vert
		glVertex3f(1.5, -1.0, 0.0);
		glColor3f(0.0, 0.0, 1.0);	// bleu
		glVertex3f(0.0, 1.0, 0.0);
	glEnd();
	
	glFlush();
}
