/* initWGL.cpp
 * Fichier contenant les fonctions ncessaires  la gestion
 * d'une fentre OpenGL sous Windows.
 *
 * Auteur: Vouriot Thierry
 * Parties de code: http://nehe.gamedev.net/, http://www.gametutorials.com
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 11 septembre 2002
 */

#include "initWGL.h"

#define WND_CLASS_NAME  "OpenGL Window Class"	// Nom de la classe de la fentre
#define WM_TOGGLEFULLSCREEN (WM_USER+1)		// message pour basculer fullscreen/windowed

static bool g_isProgramLooping;	// boucle de cration des fentres, ncessaire pour le basculement fullscreen/windowed
static bool g_createFullScreen;	// si true, alors la fentre est crer en mode plein cran

/* Termine l'application */
void TerminateApplication(GL_Window* window)	
{
	// envoi du message WM_QUIT
	PostMessage(window->hWnd, WM_QUIT, 0, 0);
	// on stoppe la boucle de l'application
	g_isProgramLooping = false;
}

/* Basculement en mode plein cran ou en mode fentre */
void ToggleFullscreen(GL_Window* window)
{
	// envoi du message WM_TOGGLEFULLSCREEN
	PostMessage(window->hWnd, WM_TOGGLEFULLSCREEN, 0, 0);
}

/* Redimensionne la taille de la zone de sortie OpenGL (viewport) */
void ReshapeOpenGLScreen(int width, int height)
{
	glViewport(0,0,(GLsizei)width,(GLsizei)height);		// Dfinie la taille de la zone de sortie
					
	glMatrixMode(GL_PROJECTION);		// Slectionne la matrice de projection
	glLoadIdentity();			// Initialise la matrice de projection
	
	//	       focale	     largeur	   hauteur   clipping min et max
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, 1.0f ,100.0f);

	glMatrixMode(GL_MODELVIEW);		// Slectionne la matrice Modelview
	glLoadIdentity();			// Initialise la matrice Modelview
}

/* Passage en mode plein cran */
bool ChangeToFullScreen(int width, int height, int bitsPerPixel)
{
	DEVMODE dmSettings;	// Device Mode variable
	ZeroMemory (&dmSettings, sizeof (DEVMODE));	// Efface la mmoire
	dmSettings.dmSize = sizeof (DEVMODE);		// taille de la structure DEVMODE
	dmSettings.dmPelsWidth = width;			// Largeur de l'cran
	dmSettings.dmPelsHeight = height;		// Hauteur de l'cran
	dmSettings.dmBitsPerPel = bitsPerPixel;		// nb de bits par pixels
	dmSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
	// Cette fonction fait passer en mode plein cran
	if (ChangeDisplaySettings(&dmSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
	{
		return false;			
	}
	return true;	
}

/* Fonction paramtrant le PixelFormat pour OpenGL */
bool SetupPixelFormat(GL_Window *window) 
{ 
    	int pixelformat; 
	PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof (PIXELFORMATDESCRIPTOR),	// taille du descripteur de format
		1,				// version
		PFD_DRAW_TO_WINDOW |		// support des fentres
		PFD_SUPPORT_OPENGL |		// support d'OpenGL
		PFD_DOUBLEBUFFER,		// support du double buffering
		PFD_TYPE_RGBA,			// mode de couleurs
		window->init.bitsPerPixel,	// profondeur des couleurs
		0, 0, 0, 0, 0, 0,		// bits de couleur ignors
		0,				// pas de tampon alpha
		0,				// shift bit ignor
		0,				// pas de tampon d'accumulation
		0, 0, 0, 0,			// accumulation bits ignors
		16,				// 16Bit Z-Buffer (Depth Buffer)  
		0,				// pas de tampon stencil
		0,				// pas de tampon auxiliaire
		PFD_MAIN_PLANE,			// Layer principal
		0,				// Rservs
		0, 0, 0				// Layer masks
	};
 
	// Dfinition du format de pixels
    	if ( (pixelformat = ChoosePixelFormat(window->hDC, &pfd)) == false ) 
    	{ 
        	return false; 
    	} 
 
	// Mise en place du format de pixels
    	if (SetPixelFormat(window->hDC, pixelformat, &pfd) == false) 
    	{ 
        	return false; 
    	} 
    	return true;
}

/* Cration et affichage de la fentre OpenGL */
bool CreateOpenGLWindow(GL_Window* window)
{
	DWORD windowStyle = WS_OVERLAPPEDWINDOW;	// style
	DWORD windowExtendedStyle = WS_EX_APPWINDOW;	// extended style

	RECT rWindow;	// Taille et coordonnes de l'cran (rectangle)
	rWindow.left	= 0;
	rWindow.right	= window->init.width;
	rWindow.top 	= 0;
	rWindow.bottom	= window->init.height;
	
	if (window->init.isFullScreen == true)	// passage en mode plein cran
	{
		if (ChangeToFullScreen(window->init.width, window->init.height, window->init.bitsPerPixel) == false)
		{
			// chec du mode plein cran
			MessageBox(HWND_DESKTOP, "Mode Fullscreen Failed.\nRunning In Windowed Mode.", "Error", MB_OK | MB_ICONEXCLAMATION);
			window->init.isFullScreen = FALSE;
		}
		else	// succs du mode plein cran
		{
			ShowCursor(false);	// cache le curseur
			windowStyle = WS_POPUP; // style popup
			windowExtendedStyle |= WS_EX_TOPMOST; // top window (couvre tout)
		}		
	}
	else	// mode fentr
	{
		// Ajuste la fentre
		AdjustWindowRectEx(&rWindow, windowStyle, 0, windowExtendedStyle);
	}

	// cration de la fentre OpenGL
	window->hWnd = CreateWindowEx(
				windowExtendedStyle,			// extended style
				window->init.application->className,	// class name
				window->init.title,			// titre
				windowStyle,				// style
				0, 0,					// position x, y
				rWindow.right - rWindow.left,	// largeur
				rWindow.bottom - rWindow.top,	// hauteur
				HWND_DESKTOP,				// le bureau est la fentre parente
				0,					// pas de menu
				window->init.application->hInstance, 	// instance de l'application
				window);
	
	// cration de la fentre russie ?			
	if (window->hWnd == 0)
	{
		return false;
	}
	
	// rcupre le device context
	window->hDC = GetDC(window->hWnd);	
	if (window->hDC == 0)
	{
		DestroyWindow(window->hWnd);
		window->hWnd = 0;
		return false;
	}
	
	// paramtrer le pixel format
	if (SetupPixelFormat(window) == false)	
	{
		ReleaseDC(window->hWnd, window->hDC);
		window->hDC = 0;
		DestroyWindow(window->hWnd);
		window->hWnd = 0;
		return false;
	}
	
	// cration du rendering context
	window->hRC = wglCreateContext(window->hDC);
	if (window->hRC == 0)
	{
		ReleaseDC(window->hWnd, window->hDC);
		window->hDC = 0;
		DestroyWindow(window->hWnd);
		window->hWnd = 0;
		return false;
	}

	// Mise en place du rendering context
	if (wglMakeCurrent(window->hDC, window->hRC) == false)
	{
		wglDeleteContext(window->hRC);	
		window->hRC = 0;
		ReleaseDC(window->hWnd, window->hDC);
		window->hDC = 0;
		DestroyWindow(window->hWnd);
		window->hWnd = 0;
		return false;
	}

	ShowWindow(window->hWnd, SW_NORMAL);	// rend la fentre visible
	window->isVisible = true;

	ReshapeOpenGLScreen(window->init.width, window->init.height); // redimensionnement de la zone OpenGL

	ZeroMemory(window->keys, sizeof (Keys)); // efface le tableau des touches

	window->lastTickCount = GetTickCount(); // rcupre le temps depuis le dmarrage du systme

	return true;
}

/* Destruction de la fentre OpenGL et libration des ressources */
bool DestroyOpenGLWindow(GL_Window* window)
{
	if (window->hWnd != 0)
	{	
		if (window->hDC != 0)
		{
			wglMakeCurrent(window->hDC, 0);	
			if (window->hRC != 0)	
			{
				wglDeleteContext(window->hRC);	
				window->hRC = 0;
			}		
			ReleaseDC(window->hWnd, window->hDC);
			window->hDC = 0;	
		}
		DestroyWindow(window->hWnd);
		window->hWnd = 0;
	}

	if (window->init.isFullScreen)
	{
		ChangeDisplaySettings(NULL,0);
		ShowCursor(true);
	}	
	return true;
}

/* Fonction grant les messages de Windows */
LRESULT CALLBACK WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// rcupre le window context
	GL_Window* window = (GL_Window*)(GetWindowLong(hWnd, GWL_USERDATA));
	
	switch (uMsg)
	{
		// commandes systme
		case WM_SYSCOMMAND:
		{
			switch (wParam)
			{
				case SC_SCREENSAVE:	// l'conomiseur d'cran ?
				case SC_MONITORPOWER:	// moniteur mode veille ?
				return 0;		// on ne fait rien
			}
			break;
		}
		return 0;

		// cration de la fentre
		case WM_CREATE:
		{
			CREATESTRUCT* creation = (CREATESTRUCT*)(lParam); // structure de la fentre
			window = (GL_Window*)(creation->lpCreateParams);
			SetWindowLong(hWnd, GWL_USERDATA, (LONG)(window));
		}
		return 0;

		// fermeture de la fentre
		case WM_CLOSE:	
			TerminateApplication(window);
		return 0;
		
		// redimmensionnement de la fentre
		case WM_SIZE:
			switch (wParam)
			{
				case SIZE_MINIMIZED:
					window->isVisible = false;
					return 0;

				case SIZE_MAXIMIZED:
					window->isVisible = true;
					ReshapeOpenGLScreen(LOWORD (lParam), HIWORD (lParam));
					return 0;	
				case SIZE_RESTORED:
					window->isVisible = true;
					ReshapeOpenGLScreen(LOWORD (lParam), HIWORD (lParam));	
					return 0;
			}
		break;

		// touche du clavier presse
		case WM_KEYDOWN:
			if ((wParam >= 0) && (wParam <= 255))
			{
				window->keys->keyDown[wParam] = true;	
				return 0;
			}
		break;

		// touche du clavier relache
		case WM_KEYUP:	
			if ((wParam >= 0) && (wParam <= 255))
			{
				window->keys->keyDown[wParam] = false;	
				return 0;
			}
		break;

		// mode plein cran ou fentr
		case WM_TOGGLEFULLSCREEN:
			g_createFullScreen = (g_createFullScreen == true) ? false : true;
			PostMessage(hWnd, WM_QUIT, 0, 0);
		break;
	}
	
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

/* Dclaration de la nouvelle classe */
bool RegisterWindowClass (Application* application)
{					
	WNDCLASSEX windowClass;
	ZeroMemory (&windowClass, sizeof (WNDCLASSEX));			// efface la mmoire
	windowClass.cbSize = sizeof (WNDCLASSEX);			// taille de la structure
	windowClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;		// redissine la fentre  chaque mouvement/redimensionnement
	windowClass.lpfnWndProc = (WNDPROC)(WindowProc);		// fonction grant les messages
	windowClass.hInstance = application->hInstance;			// instance de l'application
	windowClass.hbrBackground = (HBRUSH)(COLOR_APPWORKSPACE);	// couleur de fond	
	windowClass.hCursor = LoadCursor(NULL, IDC_ARROW);		// curseur
	windowClass.lpszClassName = application->className;		// nom de la classe
	if (RegisterClassEx (&windowClass) == 0)
	{
		return false;					
	}
	return true;
}

/* Fonction principale pour lancer la fentre Windows */
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Application application;	// application structure
	GL_Window window;		// window structure
	Keys keys;			// key structure
	bool isMessageLoopActive;	// boucle grant les messages activ ?
	MSG msg;			// message
	DWORD tickCount;		// temps coul depuis le dmarrage du systme (millisecondes)

	// application structure
	application.className = WND_CLASS_NAME;
	application.hInstance = hInstance;

	// window strucutre
	ZeroMemory(&window, sizeof (GL_Window));	// mise  zro
	window.keys = &keys;
	window.init.application	= &application;
	window.init.title = WINDOW_TITLE;
	window.init.width = SCREEN_WIDTH;
	window.init.height = SCREEN_HEIGHT;
	window.init.bitsPerPixel = COLOR_DEPTH;
	window.init.isFullScreen = true;

	ZeroMemory(&keys, sizeof (Keys));	// mise  zro
	
	// MessageBox pour le mode plein cran
	if (MessageBox(HWND_DESKTOP, "Click Yes to go to full screen (Recommended)", "Start FullScreen?", MB_YESNO | MB_ICONQUESTION) == IDNO)
	{
		window.init.isFullScreen = false;
	}

	// dclaration de la classe de la fentre
	if (RegisterWindowClass(&application) == false)
	{
		MessageBox(HWND_DESKTOP, "Error Registering Window Class!", "Error", MB_OK | MB_ICONEXCLAMATION);
		return -1;
	}

	g_isProgramLooping = true;
	g_createFullScreen = window.init.isFullScreen;
	
	// boucle principale (permet le bassculement fullscreen/windowed
	// en envoyant le message WM_QUIT et en changeant la variable g_createFullScreen)
	while (g_isProgramLooping)
	{
		// cration de la fentre
		window.init.isFullScreen = g_createFullScreen;	
		if (CreateOpenGLWindow(&window) == true)
		{
			// cration de la fentre russi
			// initialisation  ( definir par l'utilisateur)
			if (Initialize(&window, &keys) == false)
			{
				MessageBox(HWND_DESKTOP, "Error Initialize Function", "Error", MB_OK | MB_ICONEXCLAMATION);
				TerminateApplication(&window);
			}
			else
			{	// boucle grant les messages
				isMessageLoopActive = true;
				while (isMessageLoopActive == true)
				{
					// vrification des messages
					if (PeekMessage(&msg, window.hWnd, 0, 0, PM_REMOVE) != 0)
					{
						if (msg.message != WM_QUIT)
						{
							// on transmet le message si diffrent de WM_QUIT
							DispatchMessage(&msg);
						}
						else
						{
							// on arrte la boucle des messages
							// si on reoit le message WM_QUIT
							isMessageLoopActive = false;
						}
					}
					else	// pas de messages
					{
						if (window.isVisible == false)	
						{
							// si la fentre n'est pas visible
							// on attend simplement un message
							WaitMessage();
						}
						else	// fentre visible
						{
							tickCount = GetTickCount(); // rcupre le tick count
							// mise  jour ( definir par l'utilisateur)
							Update(tickCount - window.lastTickCount);
							window.lastTickCount = tickCount; // maj de lastTickCount
							DisplayScene();	// affichage de la scne ( dfinir par l'utilisateur)

							SwapBuffers(window.hDC);// echange des tampons (Double Buffering)
						}
					}
				} // boucle des messages - fin
			}

			// Deinitialisation ( definir par l'utilisateur)
			Deinitialize();	
			// destuction de la fentre
			DestroyOpenGLWindow(&window);
		}
		else	// echec de la cration de la fentre
		{
			MessageBox(HWND_DESKTOP, "Error Creating OpenGL Window", "Error", MB_OK | MB_ICONEXCLAMATION);
			g_isProgramLooping = false;
		}
	} // boucle principal - fin

	UnregisterClass(application.className, application.hInstance);
	return 0;
}						

