/* initWGL.h
 * Fichier d'entte contenant les variables globales et les prototypes
 * des fonctions ncessaires  une application OpenGL pour Windows
 *
 * Auteur: Vouriot Thierry
 * Parties de code: http://nehe.gamedev.net/, http://www.gametutorials.com
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 11 septembre 2002
 */

#ifndef _INITWGL_H
#define _INITWGL_H

// inclusion standard
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>

// proprits de la fentre
#define WINDOW_TITLE "OpenGL Window"
#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 480
#define COLOR_DEPTH 16

// structure contenant les touches du clavier presses
typedef struct {
	bool keyDown [256];
} Keys;

// informations vitales de l'application
typedef struct {
	HINSTANCE hInstance; // instance de l'application
	const char* className; // nom de la classe
} Application;

// informations ncessaires  la cration de la fentre
typedef struct {
	Application* application;	// structure Application
	char* title;			// titre de la fentre
	int width;			// largeur
	int height;			// hauteur
	int bitsPerPixel;		// bits par pixel
	bool isFullScreen;		// plein cran ?
} GL_WindowInit;

// informations vitales de la fentre
typedef struct {
	Keys* keys;		// structure Keys
	HWND hWnd;		// fentre handle
	HDC hDC;		// device context
	HGLRC hRC;		// rendering context
	GL_WindowInit init;	// structure GL_WindowInit
	bool isVisible;		// fentre visible ?
	DWORD lastTickCount;	// tick counter (temps depuis le dmarrage du systme)
} GL_Window;	

/* Termine l'application */
void TerminateApplication(GL_Window* window);

/* Basculement en mode plein cran ou en mode fentre */
void ToggleFullscreen(GL_Window* window);

/* Les fonctions suivantes doivent tre fournies par l'utilisateur */

/* Initialisation de l'utilisateur et d'OpenGL */
bool Initialize(GL_Window* window, Keys* keys);

/* Appele juste avant la destruction de la fentre,
 * Libration mmoire, ...etc... 
 */
void Deinitialize(void);

/* Appele avant chaque affichage d'image,
 * Teste des touches clavier,
 * Calcule du frame rate  (milliseconds est le temps entre 2 affichages d'images),
 * Maj des diffrentes variables, ...etc...
 */
void Update(DWORD milliseconds);

/* Affichage de la scne */
void DisplayScene(void);	

#endif 

